@@ -4,11 +4,11 @@ import json
 from base64 import urlsafe_b64decode
 import requests
 from requests.exceptions import HTTPError
-from flask import Request, jsonify, abort
+from flask import Request, jsonify
 from . import util
 
 
-SERVICE_CATALOG = os.environ["SERVICE_CATALOG"]
+SERVICE_CATALOG = json.loads(os.environ["SERVICE_CATALOG"])
 
 
 def _get_auth_token(audience):
@@ -18,7 +18,8 @@ def _get_auth_token(audience):
         f"?audience={audience}"
     )
     return requests.get(
-        url=token_url, headers={"Metadata-Flavor": "Google"},
+        url=token_url,
+        headers={"Metadata-Flavor": "Google"},
     ).content.decode()
 
 
@@ -44,16 +45,57 @@ def user_info(request: Request):
     return {}
 
 
+def handle_http_error(func):
+    def wrapper(request: Request):
+        try:
+            return func(request)
+        except HTTPError as e:
+            logging.exception(e.response.content.decode())
+        except Exception as e:
+            logging.exception(e)
+        return (
+            jsonify(message="An unexpected error occurred."),
+            500,
+        )
+
+    return wrapper
+
+
+def validate(**kwargs):
+    def wrap(func):
+        def wrapper(request: Request):
+            try:
+                for key in kwargs:
+                    valid = kwargs[key](request.json.get(key))
+                    if not valid:
+                        return (
+                            jsonify(message=f"Invalid parameter value for '{key}'"),
+                            400,
+                        )
+            except Exception as e:
+                logging.exception(e)
+                return (
+                    jsonify(message="An unexpected error occurred in validation."),
+                    500,
+                )
+            return func(request)
+
+        return wrapper
+
+    return wrap
+
+
 def auth(permission):
     def wrap(func):
-        def wrapped_func(request: Request):
+        def wrapper(request: Request):
             try:
                 if not "asset" in request.json:
-                    resp = jsonify(
-                        message="Missing required 'asset' parameter in authorized request."
+                    return (
+                        jsonify(
+                            message="Missing required 'asset' parameter in authorized request."
+                        ),
+                        400,
                     )
-                    resp.status_code = 400
-                    abort(resp)
                 r = call(
                     "db",
                     "authorize",
@@ -63,17 +105,16 @@ def auth(permission):
                 )
                 r.raise_for_status()
                 if not r.json()["granted"]:
-                    resp = jsonify(message="Access denied.")
-                    resp.status_code = 403
-                    abort(resp)
+                    return jsonify(message="Access denied."), 403
             except Exception as e:
                 logging.exception(e)
-                resp = jsonify(message="An unexpected error occurred in authorization.")
-                resp.status_code = 500
-                abort(resp)
+                return (
+                    jsonify(message="An unexpected error occurred in authorization."),
+                    500,
+                )
             return func(request)
 
-        return wrapped_func
+        return wrapper
 
     return wrap
 
@@ -96,18 +137,9 @@ class Asset:
 
 class ParsedRequest:
     def __init__(self, request: Request):
-        data = request.json
+        self.data = request.json
         self.user_info = user_info(request)
-        self.data = data
-        self.repo = util.repo_name(data.get("wid"), data.get("cid"))
-        self.change_ref = util.change_ref(
-            data.get("sid"), self.user_info.get("user_id"), data.get("change_id")
-        )
-        self.plan_ref = util.plan_ref(
-            data.get("sid"), self.user_info.get("user_id"), data.get("change_id")
-        )
-        self.apply_ref = self.apply_ref(data.get("sid"))
-        self.source_apply_ref = self.apply_ref(data.get("source_sid"))
+        self.asset = Asset(self.data["asset"]) if "asset" in self.data else None
 
 
 class DBEvent:
@@ -134,4 +166,3 @@ class DBEvent:
             value_type = next(iter(fields[key]))
             values[key] = self._raise_value(fields[value_type])
         return values
-
